@@ -2,11 +2,13 @@ from __future__ import annotations
 
 import json
 import uuid
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional
 
 import pandas as pd
 from aqueduct.artifacts import bool_artifact, numeric_artifact
 from aqueduct.artifacts import preview as artifact_utils
+from aqueduct.artifacts import system_metric
+from aqueduct.artifacts._create import create_metric_or_check_artifact
 from aqueduct.artifacts.base_artifact import BaseArtifact
 from aqueduct.constants.enums import (
     ArtifactType,
@@ -16,36 +18,22 @@ from aqueduct.constants.enums import (
     FunctionType,
     OperatorType,
 )
-from aqueduct.constants.metrics import SYSTEM_METRICS_INFO
 from aqueduct.error import AqueductError, ArtifactNeverComputedException
-from aqueduct.models.artifact import ArtifactMetadata
 from aqueduct.models.dag import DAG
-from aqueduct.models.operators import (
-    CheckSpec,
-    FunctionSpec,
-    MetricSpec,
-    Operator,
-    OperatorSpec,
-    SystemMetricSpec,
-)
-from aqueduct.utils.dag_deltas import (
-    AddOrReplaceOperatorDelta,
-    RemoveCheckOperatorDelta,
-    apply_deltas_to_dag,
-)
+from aqueduct.models.operators import CheckSpec, FunctionSpec, MetricSpec, OperatorSpec
+from aqueduct.utils.dag_deltas import RemoveCheckOperatorDelta, apply_deltas_to_dag
 from aqueduct.utils.describe import (
     get_readable_description_for_check,
     get_readable_description_for_metric,
 )
 from aqueduct.utils.function_packaging import serialize_function
-from aqueduct.utils.naming import resolve_op_and_artifact_names
-from aqueduct.utils.utils import format_header_for_print, generate_uuid
+from aqueduct.utils.utils import format_header_for_print
 from ruamel import yaml
 
 from aqueduct import globals
 
 
-class TableArtifact(BaseArtifact):
+class TableArtifact(BaseArtifact, system_metric.SystemMetricMixin):
     """This class represents a computed table within the flow's DAG.
 
     Any `@op`-annotated python function that returns a dataframe will
@@ -153,15 +141,6 @@ class TableArtifact(BaseArtifact):
         """
         return self.PRESET_METRIC_LIST
 
-    def list_system_metrics(self) -> List[str]:
-        """Returns a list of all system metrics available on the table artifact.
-        These system metrics can be set via the invoking the system_metric() method the table.
-
-        Returns:
-            A list of available system metrics on a table
-        """
-        return list(SYSTEM_METRICS_INFO.keys())
-
     def validate_with_expectation(
         self,
         expectation_name: str,
@@ -274,6 +253,8 @@ class TableArtifact(BaseArtifact):
         check_spec = OperatorSpec(check=CheckSpec(level=severity, function=function_spec))
         check_description = "Check table with built in expectations from great expectations"
         new_artifact = self._apply_operator_to_table(
+            self._dag,
+            self._artifact_id,
             check_spec,
             check_name,
             check_description,
@@ -349,6 +330,8 @@ class TableArtifact(BaseArtifact):
         )
         op_spec = OperatorSpec(metric=MetricSpec(function=function_spec))
         new_artifact = self._apply_operator_to_table(
+            self._dag,
+            self._artifact_id,
             op_spec,
             metric_name,
             metric_description,
@@ -390,6 +373,8 @@ class TableArtifact(BaseArtifact):
         )
         op_spec = OperatorSpec(metric=MetricSpec(function=function_spec))
         new_artifact = self._apply_operator_to_table(
+            self._dag,
+            self._artifact_id,
             op_spec,
             metric_name,
             metric_description,
@@ -436,6 +421,8 @@ class TableArtifact(BaseArtifact):
         )
         op_spec = OperatorSpec(metric=MetricSpec(function=function_spec))
         new_artifact = self._apply_operator_to_table(
+            self._dag,
+            self._artifact_id,
             op_spec,
             metric_name,
             metric_description,
@@ -482,6 +469,8 @@ class TableArtifact(BaseArtifact):
         )
         op_spec = OperatorSpec(metric=MetricSpec(function=function_spec))
         new_artifact = self._apply_operator_to_table(
+            self._dag,
+            self._artifact_id,
             op_spec,
             metric_name,
             metric_description,
@@ -528,6 +517,8 @@ class TableArtifact(BaseArtifact):
         )
         op_spec = OperatorSpec(metric=MetricSpec(function=function_spec))
         new_artifact = self._apply_operator_to_table(
+            self._dag,
+            self._artifact_id,
             op_spec,
             metric_name,
             metric_description,
@@ -576,6 +567,8 @@ class TableArtifact(BaseArtifact):
         )
         op_spec = OperatorSpec(metric=MetricSpec(function=function_spec))
         new_artifact = self._apply_operator_to_table(
+            self._dag,
+            self._artifact_id,
             op_spec,
             metric_name,
             metric_description,
@@ -594,8 +587,8 @@ class TableArtifact(BaseArtifact):
 
         Args:
             metric_name:
-                name of system metric to retrieve for the table.
-                valid metrics are:
+                Name of system metric to retrieve for the table.
+                Valid metrics are:
                     runtime: runtime of previous @op func in seconds
                     max_memory: maximum memory usage of previous @op func in Mb
 
@@ -604,84 +597,8 @@ class TableArtifact(BaseArtifact):
         """
         if globals.__GLOBAL_CONFIG__.lazy:
             lazy = True
-        execution_mode = ExecutionMode.EAGER if not lazy else ExecutionMode.LAZY
 
-        operator = self._dag.must_get_operator(with_output_artifact_id=self._artifact_id)
-        system_metric_description, system_metric_unit = SYSTEM_METRICS_INFO[metric_name]
-        system_metric_name = "%s %s(%s) metric" % (operator.name, metric_name, system_metric_unit)
-        op_spec = OperatorSpec(system_metric=SystemMetricSpec(metric_name=metric_name))
-        new_artifact = self._apply_operator_to_table(
-            op_spec,
-            system_metric_name,
-            system_metric_description,
-            output_artifact_type_hint=ArtifactType.NUMERIC,
-            execution_mode=execution_mode,
-        )
-
-        assert isinstance(new_artifact, numeric_artifact.NumericArtifact)
-
-        return new_artifact
-
-    def _apply_operator_to_table(
-        self,
-        op_spec: OperatorSpec,
-        op_name: str,
-        op_description: str,
-        output_artifact_type_hint: ArtifactType,
-        execution_mode: ExecutionMode = ExecutionMode.EAGER,
-    ) -> Union[numeric_artifact.NumericArtifact, bool_artifact.BoolArtifact]:
-        assert (
-            output_artifact_type_hint == ArtifactType.NUMERIC
-            or output_artifact_type_hint == ArtifactType.BOOL
-        )
-
-        operator_id = generate_uuid()
-        output_artifact_id = generate_uuid()
-
-        op_name, artifact_names = resolve_op_and_artifact_names(
-            self._dag,
-            op_name,
-            overwrite_existing_op_name=False,
-        )
-        assert len(artifact_names) == 1
-
-        apply_deltas_to_dag(
-            self._dag,
-            deltas=[
-                AddOrReplaceOperatorDelta(
-                    op=Operator(
-                        id=operator_id,
-                        name=op_name,
-                        description=op_description,
-                        spec=op_spec,
-                        inputs=[self._artifact_id],
-                        outputs=[output_artifact_id],
-                    ),
-                    output_artifacts=[
-                        ArtifactMetadata(
-                            id=output_artifact_id,
-                            name=artifact_names[0],
-                            type=output_artifact_type_hint,
-                        )
-                    ],
-                ),
-            ],
-        )
-
-        if execution_mode == ExecutionMode.EAGER:
-            # Issue preview request since this is an eager execution.
-            artifact = artifact_utils.preview_artifact(self._dag, output_artifact_id)
-
-            assert isinstance(artifact, numeric_artifact.NumericArtifact) or isinstance(
-                artifact, bool_artifact.BoolArtifact
-            )
-            return artifact
-        else:
-            # We are in lazy mode.
-            if output_artifact_type_hint == ArtifactType.NUMERIC:
-                return numeric_artifact.NumericArtifact(self._dag, output_artifact_id)
-            else:
-                return bool_artifact.BoolArtifact(self._dag, output_artifact_id)
+        return self._system_metric_helper(self._dag, self._artifact_id, metric_name, lazy)
 
     def _get_table_name(self) -> str:
         return self._dag.must_get_artifact(self._artifact_id).name
