@@ -10,6 +10,7 @@ from aqueduct.error import (
     ClientValidationError,
     InternalServerError,
     InvalidRequestError,
+    InvalidUserActionException,
     NoConnectedIntegrationsException,
     ResourceNotFoundError,
     UnprocessableEntityError,
@@ -40,6 +41,10 @@ from .response_models import (
     SavedObjectUpdate,
 )
 
+# The maximum http request size is capped at 32 MB. DAG containing
+# local data parameter(s) should not go beyond this value.
+MAX_REQUEST_BODY_SIZE = 32 << 20
+
 
 class APIClient:
     """
@@ -403,6 +408,14 @@ class APIClient:
             "dag": dag.json(exclude_none=True),
         }
 
+        if len(body["dag"]) > MAX_REQUEST_BODY_SIZE and any(
+            artifact_metadata.from_local_data for artifact_metadata in list(dag.artifacts.values())
+        ):
+            raise InvalidUserActionException(
+                "Local Data after serialization is too large. Aqueduct uses json serialization. The maximum size of workflow with local data is %s in bytes, the current size is %s in bytes."
+                % (MAX_REQUEST_BODY_SIZE, len(body["dag"]))
+            )
+
         files: Dict[str, IO[Any]] = {}
         for op in dag.list_operators():
             file = op.file()
@@ -452,6 +465,14 @@ class APIClient:
             "dag": dag.json(exclude_none=True),
         }
 
+        if len(body["dag"]) > MAX_REQUEST_BODY_SIZE and any(
+            artifact_metadata.from_local_data for artifact_metadata in list(dag.artifacts.values())
+        ):
+            raise InvalidUserActionException(
+                "Local Data after serialization is too large. Aqueduct uses json serialization. The maximum size of workflow with local data is %s in bytes, the current size is %s in bytes."
+                % (MAX_REQUEST_BODY_SIZE, len(body["dag"]))
+            )
+
         files: Dict[str, IO[Any]] = {}
         for op in dag.list_operators():
             file = op.file()
