@@ -1,15 +1,16 @@
 import inspect
+import uuid
 import warnings
 from functools import wraps
 from typing import Any, Callable, Dict, List, Mapping, Optional, Union, cast
 
 import numpy as np
+from aqueduct.artifacts._create import create_metric_or_check_artifact
 from aqueduct.artifacts.base_artifact import BaseArtifact
 from aqueduct.artifacts.bool_artifact import BoolArtifact
-from aqueduct.artifacts.create import check_implicit_param_name, create_param_artifact
+from aqueduct.artifacts.create import create_param_artifact, to_artifact_class
 from aqueduct.artifacts.numeric_artifact import NumericArtifact
 from aqueduct.artifacts.preview import preview_artifacts
-from aqueduct.artifacts.transform import to_artifact_class
 from aqueduct.constants.enums import (
     ArtifactType,
     CheckSeverity,
@@ -20,6 +21,7 @@ from aqueduct.constants.enums import (
     OperatorType,
 )
 from aqueduct.error import InvalidUserActionException, InvalidUserArgumentException
+from aqueduct.logger import logger
 from aqueduct.models.artifact import ArtifactMetadata
 from aqueduct.models.operators import (
     CheckSpec,
@@ -28,11 +30,17 @@ from aqueduct.models.operators import (
     Operator,
     OperatorSpec,
     ResourceConfig,
+    get_operator_type,
 )
 from aqueduct.type_annotations import CheckFunction, MetricFunction, Number, UserFunction
-from aqueduct.utils.dag_deltas import AddOrReplaceOperatorDelta, apply_deltas_to_dag
+from aqueduct.utils.dag_deltas import (
+    AddOperatorDelta,
+    DAGDelta,
+    RemoveOperatorDelta,
+    apply_deltas_to_dag,
+)
 from aqueduct.utils.function_packaging import serialize_function
-from aqueduct.utils.naming import resolve_op_and_artifact_names
+from aqueduct.utils.naming import default_artifact_name_from_op_name, sanitize_artifact_name
 from aqueduct.utils.utils import generate_engine_config, generate_uuid
 
 from aqueduct import globals
@@ -88,6 +96,10 @@ def wrap_spec(
             The artifact types that the function is expected to output, in the correct order.
         description:
             The description for this operator.
+        execution_mode:
+            Whether the operator should be executed eagerly or lazily.
+        overwrite_op:
+            The operator to overwrite, if any. Should only be set for metrics and checks.
 
     Returns:
         A list of artifacts, representing the outputs of the function.
@@ -112,55 +124,44 @@ def wrap_spec(
     operator_id = generate_uuid()
     output_artifact_ids = [generate_uuid() for _ in output_artifact_type_hints]
 
-    overwrite_existing_op = True
-    colliding_op = dag.get_operator(with_name=op_name)
-    if colliding_op is not None:
-        # There is already an operator with the same name.
-        # If the input artifacts are different for this new operator
-        # and the existing operator, then a new operator should be added.
-        # Otherwise, the existing operator should be overwritten.
-        input_artifact_ids = [input_artf.id() for input_artf in input_artifacts]
-        if input_artifact_ids != colliding_op.inputs:
-            overwrite_existing_op = False
+    # Even if there are multiple outputs, we give them all the same artifact names (in the default case).
+    # These will be deduplicated at publish time.
+    artifact_names = output_artifact_names or [
+        default_artifact_name_from_op_name(op_name) for _ in range(len(output_artifact_ids))
+    ]
 
-    original_op_name = op_name
-    op_name, artifact_names = resolve_op_and_artifact_names(
-        dag,
-        op_name,
-        overwrite_existing_op_name=overwrite_existing_op,
-        candidate_artifact_names=output_artifact_names,
-        num_outputs=len(output_artifact_ids),
+    new_op = Operator(
+        id=operator_id,
+        name=op_name,
+        description=description,
+        spec=spec,
+        inputs=[artifact.id() for artifact in input_artifacts],
+        outputs=output_artifact_ids,
     )
 
-    if colliding_op is not None and op_name != original_op_name:
-        print(
-            "Warning: There is already an operator named %s so this operator is being renamed to %s"
-            % (original_op_name, op_name)
+    new_output_artifacts = [
+        ArtifactMetadata(
+            id=output_artifact_id,
+            name=sanitize_artifact_name(artifact_names[i]),
+            type=output_artifact_type_hints[i],
+            explicitly_named=output_artifact_names is not None,
         )
+        for i, output_artifact_id in enumerate(output_artifact_ids)
+    ]
 
-    apply_deltas_to_dag(
-        dag,
-        deltas=[
-            AddOrReplaceOperatorDelta(
-                op=Operator(
-                    id=operator_id,
-                    name=op_name,
-                    description=description,
-                    spec=spec,
-                    inputs=[artifact.id() for artifact in input_artifacts],
-                    outputs=output_artifact_ids,
-                ),
-                output_artifacts=[
-                    ArtifactMetadata(
-                        id=output_artifact_id,
-                        name=artifact_names[i],
-                        type=output_artifact_type_hints[i],
-                    )
-                    for i, output_artifact_id in enumerate(output_artifact_ids)
-                ],
-            ),
-        ],
-    )
+    # Update the dag to reflect the newly created operator.
+    if get_operator_type(new_op) in [OperatorType.METRIC, OperatorType.CHECK]:
+        create_metric_or_check_artifact(dag, new_op, new_output_artifacts)
+    else:
+        apply_deltas_to_dag(
+            dag,
+            deltas=[
+                AddOperatorDelta(
+                    op=new_op,
+                    output_artifacts=new_output_artifacts,
+                )
+            ],
+        )
 
     if execution_mode == ExecutionMode.EAGER:
         # Issue preview request since this is an eager execution.
@@ -286,10 +287,6 @@ def _convert_input_arguments_to_parameters(
     dag = globals.__GLOBAL_DAG__
     fn_param_names = list(func_params.keys())
 
-    # For each implicit parameter, record it's input index and parameter name after validation.
-    implicit_param_name_by_index: Dict[int, str] = {}
-
-    # The first pass just checks that new implicit parameters have a valid name.
     artifacts = list(input_artifacts)
     for idx, artifact in enumerate(artifacts):
         if not isinstance(artifact, BaseArtifact):
@@ -303,31 +300,50 @@ def _convert_input_arguments_to_parameters(
             # We assume that the user-function's parameter name exists here, since we've disallowed any variable-length parameters.
             # An implicit parameter will have the operator name prepended to it.
             param_name = op_name + ":" + fn_param_names[idx]
-            is_overwrite = check_implicit_param_name(
-                dag,
-                param_name,
-                op_name,
+
+            # Implicit parameters are only ever created (or error). They never replace anything.
+            logger().warning(
+                """Operator `%s`'s argument `%s` is not an artifact type. We have implicitly created a parameter named `%s` and your input will be used as its default value. This parameter will be used when running the function."""
+                % (op_name, fn_param_names[idx], param_name)
             )
-            if is_overwrite:
-                warnings.warn(
-                    """Input to function argument `%s` is not an artifact type. We have implicitly created a parameter named `%s` and your input will be used as its default value. This parameter will be used when running the function."""
-                    % (param_name, param_name)
-                )
-
-            implicit_param_name_by_index[idx] = param_name
-
-    # Take a second pass to actually create the artifacts and modify the DAG.
-    for idx, artifact in enumerate(artifacts):
-        if idx in implicit_param_name_by_index.keys():
-            assert not isinstance(artifact, BaseArtifact)
-            default = artifact
-
             artifacts[idx] = create_param_artifact(
                 dag=dag,
-                param_name=implicit_param_name_by_index[idx],
-                default=default,
-                is_implicit=True,
+                param_name=param_name,
+                default=artifact,
+                description="Parameter corresponding to argument `%s` of function `%s`."
+                % (fn_param_names[idx], op_name),
+                explicitly_named=False,
             )
+
+    # If the user has supplied fewer arguments than the function takes, we check if the remaining
+    # arguments have default values. If they do, we create implicit parameters for them.
+    if len(artifacts) < len(fn_param_names):
+        for idx in range(len(artifacts), len(fn_param_names)):
+            default_value = func_params[fn_param_names[idx]].default
+            if default_value is inspect.Parameter.empty:
+                raise InvalidUserArgumentException(
+                    """No input was provided for argument `%s` of function `%s`, and no default value was specified."""
+                    % (fn_param_names[idx], op_name)
+                )
+
+            param_name = op_name + ":" + fn_param_names[idx]
+
+            # Implicit parameters are only ever created (or error). They never replace anything.
+            logger().warning(
+                """Operator `%s`'s argument `%s` is not an artifact type. We have implicitly created a parameter named `%s` and your input will be used as its default value. This parameter will be used when running the function."""
+                % (op_name, fn_param_names[idx], param_name)
+            )
+            artifacts.append(
+                create_param_artifact(
+                    dag=dag,
+                    param_name=param_name,
+                    default=default_value,
+                    description="Parameter corresponding to argument `%s` of function `%s`."
+                    % (fn_param_names[idx], op_name),
+                    explicitly_named=False,
+                )
+            )
+
     return artifacts
 
 
@@ -466,8 +482,7 @@ def op(
             Can be either a path to the requirements.txt file or a list of pip requirements specifiers.
             (eg. ["transformers==4.21.0", "numpy==1.22.4"]. If not supplied, we'll first
             look for a `requirements.txt` file in the same directory as the decorated function
-            and install those. Otherwise, we'll attempt to infer the requirements with
-            `pip freeze`.
+            and install those. Otherwise, the method raises RequirementsMissingError exception.
         num_outputs:
             The number of outputs the decorated function is expected to return.
             Will fail at runtime if a different number of outputs is returned by the function.
@@ -653,8 +668,7 @@ def metric(
             Can be either a path to the requirements.txt file or a list of pip requirements specifiers.
             (eg. ["transformers==4.21.0", "numpy==1.22.4"]. If not supplied, we'll first
             look for a `requirements.txt` file in the same directory as the decorated function
-            and install those. Otherwise, we'll attempt to infer the requirements with
-            `pip freeze`.
+            and install those. Otherwise, the method raises RequirementsMissingError exception.
         output:
             An optional custom name for the output metric artifact. Otherwise, the default naming scheme
             will be used.
@@ -838,8 +852,7 @@ def check(
             Can be either a path to the requirements.txt file or a list of pip requirements specifiers.
             (eg. ["transformers==4.21.0", "numpy==1.22.4"]. If not supplied, we'll first
             look for a `requirements.txt` file in the same directory as the decorated function
-            and install those. Otherwise, we'll attempt to infer the requirements with
-            `pip freeze`.
+            and install those. Otherwise, the method raises RequirementsMissingError exception.
         output:
             An optional custom name for the output metric artifact. Otherwise, the default naming scheme
             will be used.
