@@ -13,10 +13,10 @@ from aqueduct.models.operators import (
     Operator,
     OperatorSpec,
 )
-from aqueduct.utils.dag_deltas import AddOrReplaceOperatorDelta, apply_deltas_to_dag
+from aqueduct.utils.dag_deltas import AddOperatorDelta, apply_deltas_to_dag
 from aqueduct.utils.utils import generate_uuid
 
-from .naming import _resolve_op_and_artifact_name_for_extract
+from ..utils.naming import default_artifact_name_from_op_name, sanitize_artifact_name
 from .save import _save_artifact
 
 
@@ -55,19 +55,15 @@ class GoogleSheetsIntegration(Integration):
         """
         integration_info = self._metadata
 
-        op_name, artifact_name = _resolve_op_and_artifact_name_for_extract(
-            dag=self._dag,
-            op_name=name,
-            default_op_name="%s query" % self.name(),
-            artifact_name=output,
-        )
+        op_name = name or "%s query" % self.name()
+        artifact_name = output or default_artifact_name_from_op_name(op_name)
 
         operator_id = generate_uuid()
         output_artifact_id = generate_uuid()
         apply_deltas_to_dag(
             self._dag,
             deltas=[
-                AddOrReplaceOperatorDelta(
+                AddOperatorDelta(
                     op=Operator(
                         id=operator_id,
                         name=op_name,
@@ -86,8 +82,9 @@ class GoogleSheetsIntegration(Integration):
                     output_artifacts=[
                         ArtifactMetadata(
                             id=output_artifact_id,
-                            name=artifact_name,
+                            name=sanitize_artifact_name(artifact_name),
                             type=ArtifactType.TABLE,
+                            explicitly_named=name is not None,
                         ),
                     ],
                 )
