@@ -1,5 +1,6 @@
+import copy
 import uuid
-from typing import Dict, List, Optional
+from typing import Any, Callable, Dict, List, Optional, Set
 
 from aqueduct.constants.enums import (
     ArtifactType,
@@ -15,6 +16,8 @@ from aqueduct.error import (
 )
 from pydantic import BaseModel
 
+from ..logger import logger
+from ..utils.naming import bump_artifact_suffix
 from .artifact import ArtifactMetadata
 from .config import EngineConfig
 from .dag_rules import check_customized_resources_are_supported
@@ -47,21 +50,80 @@ class DAG(BaseModel):
     operators: Dict[str, Operator] = {}
     artifacts: Dict[str, ArtifactMetadata] = {}
 
-    # Allows for quick operator lookup by name.
-    # Is excluded from json serialization.
-    operator_by_name: Dict[str, Operator] = {}
-
     # The field must be set when publishing the workflow.
     metadata: Metadata
 
-    # Represents the default engine the DAG will be executed on. Can we overwritten
+    # Represents the default engine the DAG will be executed on. Can be overwritten
     # by individual operators.
     engine_config: EngineConfig = EngineConfig()
 
-    class Config:
-        fields = {
-            "operator_by_name": {"exclude": ...},
-        }
+    def validate_and_resolve_artifact_names(self) -> None:
+        """To be called from publish_flow() only.
+
+        Checks that all explicitly named artifacts are unique.
+
+        For implicitly named artifacts that collide, bumps their names using the `(num)` suffix.
+        The index is grouped by the input operator, so an operator with multiple output artifacts
+        that use our default naming scheme will always have consecutive numbers. Eg. (1), (2), (3)
+        """
+
+        # In the first pass, check that there aren't any explicitly named artifacts that collide with each other.
+        # Add any explicitly named artifacts to `seen_artifact_names`. Those names are now completely claimed.
+        seen_artifact_names: Set[str] = set()
+        for artifact in self.artifacts.values():
+            if artifact.explicitly_named:
+                if artifact.name in seen_artifact_names:
+                    raise InvalidUserActionException(
+                        "Unable to publish flow. You are attempting to publish multiple artifacts explicitly named `%s`. "
+                        "Please use `artifact.set_name(<new name>)` to resolve this naming collision. Or rerun the operators "
+                        "with different output artifact names." % artifact.name
+                    )
+                seen_artifact_names.add(artifact.name)
+
+        # In the second pass, resolve the names of any implicitly named artifacts that collide. Loop through
+        # the operators in topological order so that numbers are assigned in a reasonable fashion. Output artifacts
+        # of the same operator are always numbered consecutively.
+        # For some determinism around naming, we sort the starting operators by name and then id.
+        q: List[Operator] = sorted(
+            [op for op in self.operators.values() if len(op.inputs) == 0],
+            key=lambda op: (op.name, op.id),
+        )
+        seen_op_ids: List[uuid.UUID] = []
+
+        any_op_was_renamed = False
+        while len(q) > 0:
+            curr_op = q.pop(0)
+
+            # Only traverse operators you haven't seen before.
+            if curr_op.id in seen_op_ids:
+                continue
+            seen_op_ids.append(curr_op.id)
+
+            output_artifacts = self.must_get_artifacts(curr_op.outputs)
+            for artifact in output_artifacts:
+                # Skip name resolution for explicitly named artifacts.
+                # We've already checked in the first pass.
+                if not artifact.explicitly_named:
+                    # Find an unallocated name for each artifact.
+                    original_name = artifact.name
+                    while artifact.name in seen_artifact_names:
+                        artifact.name = bump_artifact_suffix(artifact.name)
+
+                    if original_name != artifact.name:
+                        logger().warning(
+                            "Multiple artifacts were named `%s`. Since artifact names must be unique, "
+                            "we renamed one of them to `%s`." % (original_name, artifact.name)
+                        )
+                        any_op_was_renamed = True
+
+                    seen_artifact_names.add(artifact.name)
+
+                q.extend(self.list_operators(on_artifact_id=artifact.id))
+
+        if any_op_was_renamed:
+            logger().warning(
+                "Note that any artifacts you explicitly gave a name to were not renamed."
+            )
 
     def set_engine_config(
         self,
@@ -127,44 +189,71 @@ class DAG(BaseModel):
     def must_get_operator(
         self,
         with_id: Optional[uuid.UUID] = None,
-        with_name: Optional[str] = None,
         with_output_artifact_id: Optional[uuid.UUID] = None,
     ) -> Operator:
-        op = self.get_operator(with_id, with_name, with_output_artifact_id)
+        op = self.get_operator(with_id, with_output_artifact_id)
         if op is None:
             raise InternalAqueductError(
-                "Unable to find operator: with_id %s, with_name %s, with_output_artifact_id %s"
-                % (str(with_id), with_name, str(with_output_artifact_id)),
+                "Unable to find operator: with_id %s, with_output_artifact_id %s"
+                % (str(with_id), str(with_output_artifact_id)),
             )
         return op
 
     def get_operator(
         self,
         with_id: Optional[uuid.UUID] = None,
-        with_name: Optional[str] = None,
         with_output_artifact_id: Optional[uuid.UUID] = None,
+        with_input_artifact_ids: Optional[List[uuid.UUID]] = None,
     ) -> Optional[Operator]:
-        if (
-            int(with_id is not None)
-            + int(with_name is not None)
-            + int(with_output_artifact_id is not None)
+        if (int(with_id is not None) + int(with_output_artifact_id is not None)) + int(
+            with_input_artifact_ids is not None
         ) != 1:
             raise InternalAqueductError(
-                "Cannot fetch operator with multiple search parameters set."
+                "Cannot fetch operator with zero or multiple search parameters set."
             )
 
         if with_id is not None:
             return self.operators.get(str(with_id))
+        elif with_output_artifact_id:
+            # Search with output artifact id
+            for _, op in self.operators.items():
+                if with_output_artifact_id in op.outputs:
+                    return op
+        elif with_input_artifact_ids:
+            for _, op in self.operators.items():
+                if set(with_input_artifact_ids) == set(op.inputs):
+                    return op
 
-        elif with_name is not None:
-            return self.operator_by_name.get(with_name)
-
-        # Search with output artifact id
-        for _, op in self.operators.items():
-            if with_output_artifact_id in op.outputs:
-                return op
         return None
 
+    def get_colliding_metric_or_check(self, candidate_op: Operator) -> Optional[Operator]:
+        """A metric or check is considered to be colliding only if it has the same name and input artifacts
+        as another metric or check. Metrics can only collide with other metrics, and checks can only collide
+        with checks.
+
+        Assumes that only one of these collisions exists.
+        """
+        assert get_operator_type(candidate_op) in [
+            OperatorType.CHECK,
+            OperatorType.METRIC,
+            OperatorType.SYSTEM_METRIC,
+        ]
+
+        match = [
+            op
+            for op in self.operators.values()
+            if (
+                get_operator_type(op) == get_operator_type(candidate_op)
+                and set(op.inputs) == set(candidate_op.inputs)
+                and op.name == candidate_op.name
+            )
+        ]
+        assert len(match) < 2, (
+            "We should not be having multiple %s's with the same name and input artifacts."
+            % get_operator_type(candidate_op)
+        )
+        return match[0] if len(match) == 1 else None
+
     def list_operators(
         self,
         filter_to: Optional[List[OperatorType]] = None,
@@ -238,7 +327,11 @@ class DAG(BaseModel):
     def must_get_artifact(self, artifact_id: uuid.UUID) -> ArtifactMetadata:
         artifact = self.get_artifact(artifact_id)
         if artifact is None:
-            raise ArtifactNotFoundException("Unable to find artifact.")
+            raise ArtifactNotFoundException(
+                "Artifact has been overwritten and no longer exists. This happens with "
+                "metrics and checks when a metric/check with the same name is run on the "
+                "same input artifacts. If this is not the case, please file a bug on us!"
+            )
         return artifact
 
     def get_artifact(self, artifact_id: uuid.UUID) -> Optional[ArtifactMetadata]:
@@ -278,15 +371,6 @@ class DAG(BaseModel):
 
         return artifacts
 
-    def validate_artifact_name(self, name: str) -> None:
-        """Checks that the artifact name is unique."""
-        existing = self.get_artifact_by_name(name)
-        if existing is not None:
-            raise InvalidUserActionException(
-                "Artifact with name `%s` has already been created locally. Artifact names must be unique."
-                % name,
-            )
-
     def list_metrics_for_operator(self, op: Operator) -> List[Operator]:
         """Returns all the metric operators on the given operator's outputs."""
         metric_operators = []
@@ -311,6 +395,12 @@ class DAG(BaseModel):
             )
         return check_operators
 
+    def get_param_op_by_name(self, name: str) -> Optional[Operator]:
+        for op in self.operators.values():
+            if op.name == name and get_operator_type(op) == OperatorType.PARAM:
+                return op
+        return None
+
     ######################## DAG WRITES #############################
 
     def add_operator(self, op: Operator) -> None:
@@ -319,7 +409,6 @@ class DAG(BaseModel):
     def add_operators(self, ops: List[Operator]) -> None:
         for op in ops:
             self.operators[str(op.id)] = op
-            self.operator_by_name[op.name] = op
 
     def add_artifacts(self, artifacts: List[ArtifactMetadata]) -> None:
         for artifact in artifacts:
@@ -329,35 +418,26 @@ class DAG(BaseModel):
         self.must_get_artifact(artifact_id).type = artifact_type
 
     def update_artifact_name(self, artifact_id: uuid.UUID, new_name: str) -> None:
-        self.must_get_artifact(artifact_id).name = new_name
-
-    def update_operator_name(self, op_id: uuid.UUID, new_name: str) -> None:
-        # Update the name -> operator map.
-        old_name = self.must_get_operator(op_id).name
-        self.operator_by_name[new_name] = self.operator_by_name[old_name]
-        del self.operator_by_name[old_name]
-
-        # Update the name on the operator spec.
-        self.must_get_operator(op_id).name = new_name
-
-    def update_operator_spec(self, name: str, spec: OperatorSpec) -> None:
-        """Replaces an operator's spec in the dag.
-
-        The assumption validated within the method is that the caller has already validated
-        both that the operator exists, and that the spec type will be unchanged.
+        """Updates an artifact to have a user-specified name. This means the artifact is
+        now explicitly named.
         """
-        assert name in self.operator_by_name, "Operator %s does not exist." % name
-        op = self.operator_by_name[name]
-        assert get_operator_type(op) == get_operator_type_from_spec(
-            spec
-        ), "New spec has a different type."
+        artifact = self.must_get_artifact(artifact_id)
+        artifact.name = new_name
+        artifact.explicitly_named = True
 
-        self.operators[str(op.id)].spec = spec
-        self.operator_by_name[op.name].spec = spec
+    def update_param_spec(self, name: str, new_spec: OperatorSpec) -> None:
+        """Checks that:
+        1) The parameter already exists, and there is not more than one with the same name.
+        2) The spec is a parameter spec.
 
-    def update_operator_function(self, operator: Operator, serialized_function: bytes) -> None:
-        if operator in self.operators.values():
-            operator.update_serialized_function(serialized_function)
+        The new parameter's value must also be of the same type, but we enforce during execution.
+        """
+        assert get_operator_type_from_spec(new_spec) == OperatorType.PARAM
+        param_op = self.get_param_op_by_name(name)
+        assert param_op is not None
+        assert get_operator_type(param_op) == OperatorType.PARAM
+
+        self.operators[str(param_op.id)].spec = new_spec
 
     def remove_operator(
         self,
@@ -391,4 +471,3 @@ class DAG(BaseModel):
             for artifact_id in op_to_remove.outputs:
                 del self.artifacts[str(artifact_id)]
             del self.operators[str(op_to_remove.id)]
-            del self.operator_by_name[op_to_remove.name]
